/*
 * 21:57:18 - 25/05/2002
 *
 * JextLauncher.cpp - A windows launcher for Jext or any Java app
 * Copyright (C) 2001-2002 Romain Guy
 * romain.guy@jext.org
 * www.jext.org
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//////////////////////////////////////////////////////////////////
// INCLUDES
//////////////////////////////////////////////////////////////////
// misc
//#define UNICODE //define it by options(alt-f7) dialog for WinNT/2000/XP
#ifdef UNICODE
#define _UNICODE
#else
#undef _UNICODE
#endif


// registry and shell
#include <windows.h>
//#include <stdlib.h>
#include <shlwapi.h>
//#include <stdio.h>
#include <shellapi.h>
#include <io.h>                   //for finding files
#include <tchar.h>
#include <assert.h>
#include <vector>                 //This should require MSVCP60.DLL, but actually it doesn't,
                                  //since everything is inlined.
using namespace std;
#ifdef _MSC_VER                   //Visual C++ 5.0/6.0 at least, I hope it works with other ones
#define fullpath _fullpath        //for compatibility with VC++
#endif
//////////////////////////////////////////////////////////////////
// HEADER
//////////////////////////////////////////////////////////////////

void    ExecuteJext(LPCTSTR path,LPCTSTR lpCmdLine);
TCHAR** GetClassPath(int *length/*, bool expandEnvVars = true*/); //2nd parameter seems useless
bool    IsJextInstalled();
//bool    Launch(LPTSTR key, LPTSTR lpCmdLine);
bool    Launch(LPCTSTR keys[], int nKeys, LPTSTR lpCmdLine);
bool    JPathFileExists(LPCTSTR fileName);

//////////////////////////////////////////////////////////////////
// CLASSES
//////////////////////////////////////////////////////////////////

// reads informations from INI files

class INIReader
{
// PRIVATE FIELDS
private:
  TCHAR iniPath[MAX_PATH];

// PUBLIC FIELDS
public:

  INIReader(TCHAR fileName[])
  {
    // constructs path to INI file
    //GetCurrentDirectory((DWORD) MAX_PATH, iniPath);
    //_tcscat(iniPath, _T("\\"));
    GetModuleFileName(NULL, iniPath, (DWORD) MAX_PATH);
    //this works and is better, why comment it out?
    TCHAR *lastSlash = _tcsrchr(iniPath, _T('\\'));
    //if (lastSlash)
    assert(lastSlash);  //the path should always contain a slash!
    *(++lastSlash) = NULL; 
    //else
      //_tcscat(iniPath, _T("\\"));                   //I don't understand this
    _tcscat(iniPath, fileName);
  };

  LPTSTR GetKeyValue(LPCTSTR lpSection, LPCTSTR lpKey, LPCTSTR lpDefault, LPTSTR outStr);
};

LPTSTR INIReader::GetKeyValue(LPCTSTR lpSection, LPCTSTR lpKey, LPCTSTR lpDefault, LPTSTR outStr)
{
  GetPrivateProfileString(lpSection, lpKey, lpDefault, outStr, (DWORD)MAX_PATH, iniPath);
  return outStr;
};

// reads registry informations

class RegistryReader
{
// PUBLIC FIELDS
public:

  // Retrieve a string value. If the given buffer for the string is too small (specified
  // by rdwSize), rdwSize is increased to the correct value. If the buffer is bigger than
  // the retrieved string, rdwSize is set to the length of the string (in TCHARs) including
  // the terminating null.
  static bool GetStringValue(HKEY hKeyRoot, LPCTSTR pszSubKey, LPCTSTR pszValue, LPTSTR pszBuffer, DWORD& rdwSize);
};

bool RegistryReader::GetStringValue(HKEY hKeyRoot, LPCTSTR pszSubKey, LPCTSTR pszValue,
                                    LPTSTR pszBuffer, DWORD& rdwSize)
{
  HKEY hKey;
  DWORD dwType = REG_SZ;
  LONG  lRes;
#ifdef UNICODE
  rdwSize *= 2;                                 //see under
#endif

  if (!pszBuffer)
    return false;

  if ((lRes = RegOpenKeyEx(hKeyRoot, pszSubKey, 0, KEY_READ, &hKey))!=ERROR_SUCCESS)
  {
    SetLastError(lRes);
    return false;
  }
	        
  // the cast to BYTE* is valid, but we need to specify, by rdwSize, 
  // the size in byte of the var, that, if UNICODE is defined, is twice
  // the declared size(a TCHAR in this case is 2 bytes)
  lRes = RegQueryValueEx(hKey, pszValue, NULL, &dwType, (BYTE*) pszBuffer, &rdwSize);

  RegCloseKey(hKey);
#ifdef UNICODE
  rdwSize /= 2;
#endif

  if (dwType != REG_SZ)
    return false;
  
  if (lRes != ERROR_SUCCESS)
  {
    SetLastError(lRes);
    return false;
  }

  return true;
};

//////////////////////////////////////////////////////////////////
// FIELDS
//////////////////////////////////////////////////////////////////
INIReader reader(_T("jextlauncher.ini"));
LPTSTR * classPaths;
int numOfClPaths;

//////////////////////////////////////////////////////////////////
// METHODS AND FUNCTIONS
//////////////////////////////////////////////////////////////////
//finds if a file exists
bool JPathFileExists(LPCTSTR fileName)
{
  /*FILE * f = _tfopen(fileName, "r");
  if (f == NULL)
    return false;
  fclose(f);
  return true;*/
  _finddata_t data;
  long handle = _tfindfirst(fileName, &data);
  if (handle == -1)
    return false;
  else {
    _findclose(handle);
    return true;
  }
}

// execute Jext
void ExecuteJext(LPCTSTR path, LPCTSTR lpCmdLine)
{

  // javaw arguments string
  TCHAR arguments[MAX_PATH * 9];
  // GetKeyValue output strings;
  TCHAR outStr[MAX_PATH];
  // flags
  reader.GetKeyValue(_T("Launcher"), _T("flags"), _T("-Xms32m"), arguments);
  _tcscat(arguments, _T(" "));

  // get current executing path

  TCHAR executionPath[MAX_PATH];
  GetModuleFileName(NULL, executionPath, MAX_PATH);
  PathRemoveFileSpec(executionPath);

  // classpath
  if (numOfClPaths > 0)
  {
    _tcscat(arguments, _T("-classpath "));
    TCHAR outPath[MAX_PATH];
    for (int i = 0; i < numOfClPaths; i++)
    {
      PathCombine(outPath, executionPath, classPaths[i]);
      //fullpath(outPath, classPaths[i], MAX_PATH);
      
      // add the classpath[i] to the command line
      _tcscat(arguments, _T("\""));
      _tcscat(arguments, outPath);
      _tcscat(arguments, _T("\""));

      if (i != numOfClPaths - 1)
        _tcscat(arguments, _T(";"));
    }
  }

  // main class
  _tcscat(arguments, _T(" "));
  _tcscat(arguments, reader.GetKeyValue(_T("Launcher"), _T("main-class"), _T("org.jext.Jext"), outStr));

  // arguments
  _tcscat(arguments, _T(" "));
  _tcscat(arguments, lpCmdLine);

  // execute
#ifdef _DEBUG
  TCHAR Message[2000];
  _tcscpy(Message, _T("The command is: \""));
  _tcscat(Message, path);
  _tcscat(Message, _T(" "));
  _tcscat(Message, arguments);
  _tcscat(Message, _T("\"\nand it is executed from the path: "));
  _tcscat(Message, executionPath);
  int choice = MessageBox(NULL,
               Message,
               reader.GetKeyValue(_T("Messages"), _T("launcher.title"), _T("Launcher"), outStr),
               MB_ICONEXCLAMATION || MB_OKCANCEL);
  if (choice != IDCANCEL) //so in Debug mode ShellExecute is called only
      // if the user doesn't press Cancel
#endif
  ShellExecute(0, _T("open"),
                  path,
                  arguments,
                  executionPath,
                  SW_SHOWNORMAL);
}

// launch Jext using the <key> Java runtime

//bool Launch(LPTSTR key, LPTSTR lpCmdLine)
bool Launch(LPCTSTR keys[], const int nKeys, LPTSTR lpCmdLine)
{
  // path
  LPCTSTR PATH = _T("Software\\JavaSoft\\");

  // buffer
  TCHAR buffer[MAX_PATH];
  DWORD bufferSize = MAX_PATH;
  // misc buffers
  TCHAR pathBuffer[100];                     //Huge enough, actual maximum will be less than 50
  TCHAR readKeyName[10], keyName[10];

  // Read user supplied version; for each user supplied one, tries in every item of keys[]
  //it now works this way since otherwise it doesn't find the JDK 1.3 because
  //there is the JRE 1.4

  LPCTSTR cJavaVersionPref = _T("JavaVersion.");
  //LPTSTR javaVersion = new TCHAR[_tcslen(cJavaVersionPref) + 3];//you also forgot to delete[] it!
  TCHAR javaVersion[20];
  TCHAR increment[2];

  for (int nJavaVersion = 0; nJavaVersion < 10; nJavaVersion++) {
    _itot(nJavaVersion, increment, 10);
    _tcscpy(javaVersion, cJavaVersionPref);
    _tcscat(javaVersion, increment);

    reader.GetKeyValue(_T("Launcher"), javaVersion, _T("0.0"), keyName);

    if ( _tcscmp(keyName, "0.0") ) {               //(using != for strings was wrong)
      // creates new key path
      //delete[] pathBuffer;
      //pathBuffer = new TCHAR[_tcslen(PATH) + _tcslen(key) + _tcslen(keyName) + 1];
      for (int currKeyI = 0; currKeyI < nKeys; currKeyI++) {
        LPCTSTR key = keys[currKeyI];
        _tcscpy(pathBuffer, PATH);
        _tcscat(pathBuffer, key);
        _tcscat(pathBuffer, keyName);

        // find JRE home
        bufferSize = MAX_PATH;
        if (RegistryReader::GetStringValue(HKEY_LOCAL_MACHINE,
                                           pathBuffer, _T("JavaHome"),
                                           buffer, bufferSize)) {
          // creates the full path
          TCHAR tmp[6];
          reader.GetKeyValue(_T("Launcher"), _T("UseJavaInterp"), _T(""), tmp);
          if ( _totlower(tmp[0]) == _T('y') )
            _tcscat(buffer, _T("\\bin\\java.exe"));
          else
            _tcscat(buffer, _T("\\bin\\javaw.exe"));

          if (JPathFileExists(buffer)) {
            ExecuteJext(buffer, lpCmdLine);
            return true;
          }
        }
      }//for end
    }
    else
      break;

    //} while (keyName[0]);//wrong, keyName[0] is always different from null; when
    //the string is missing it reads the default, 0.0
  }//outer for end
  //Version chosen in the .ini file invalid, autodetect, for each item of keys[]
  HKEY handle;
  for (int currKeyI = 0; currKeyI < nKeys; currKeyI++) {
    LPCTSTR key = keys[currKeyI];
    _tcscpy(pathBuffer, PATH);                  //With this 3 lines the autodetection should work
    _tcscat(pathBuffer, key);
    _tcscpy(keyName, _T("0.0"));
    RegOpenKeyEx(HKEY_LOCAL_MACHINE, pathBuffer, 0, KEY_READ, &handle);
    for (int i = 0;;i++) {
      if (RegEnumKey(handle, i, readKeyName, 10) != ERROR_SUCCESS)
        break;
      else if (_tcslen(readKeyName) == 3) { //main version key, not such as 1.3.1_01
        //compares version numbers
        //major & minor version numbers
        int maj    = readKeyName[0] - '0', min    = readKeyName[2] - '0',
            newMaj = keyName[0] - '0',     newMin = keyName[2] - '0';
        if (maj > newMaj || (maj == newMaj && min > newMin))
          _tcscpy(keyName, readKeyName);
      }
    }
    RegCloseKey(handle);


    //now keyName is the latest JDK/JRE version

//  if (RegistryReader::GetStringValue(HKEY_LOCAL_MACHINE,
//                                     pathBuffer, _T("CurrentVersion"),
//                                     buffer, bufferSize))
    if ( _tcscmp(keyName, "0.0") ) {                   //found any JDK/JRE
      // creates new key path
      //delete[] pathBuffer;
      //pathBuffer = new TCHAR[_tcslen(PATH) + _tcslen(key) + _tcslen(keyName) + 1];
      _tcscpy(pathBuffer, PATH);
      _tcscat(pathBuffer, key);
      _tcscat(pathBuffer, keyName);

      // find JRE home
      bufferSize = MAX_PATH;
      if (RegistryReader::GetStringValue(HKEY_LOCAL_MACHINE,
                                         pathBuffer, _T("JavaHome"),
                                         buffer, bufferSize)) {
        // creates the full path
        TCHAR tmp[6];
        reader.GetKeyValue(_T("Launcher"), _T("UseJavaInterp"), _T(""), tmp);
        if ( _totlower(tmp[0]) == _T('y') )
          _tcscat(buffer, _T("\\bin\\java.exe"));
        else
          _tcscat(buffer, _T("\\bin\\javaw.exe"));
        if (JPathFileExists(buffer)) {
          ExecuteJext(buffer, lpCmdLine);
          //delete[] pathBuffer;
          return true;
        }
      }
    }
  } //end of 2 for

  //delete[] pathBuffer;
  return false;
}

// returns the class path as a table of LPTSTR

TCHAR** GetClassPath(int *length)
{
  const int Size = MAX_PATH * 8;

  LPCTSTR cPathPref = _T("classpath.");
  //LPTSTR classpath = new TCHAR[_tcslen(cPathPref) + 3];
  TCHAR classpath[20];
  TCHAR increment[3];                                     //to contain 2 digits

  vector<LPTSTR> resPaths;
  resPaths.reserve(20);

  //here goes the string it reads, then it is split into lpPaths
  TCHAR _thisPath[Size];
  TCHAR * thisPath;

  for (int toRead = 0; toRead < 100; toRead++) {

    thisPath = _thisPath;  //since I move thisPath on the string, I need to do this

            // Builds "classpath.n" string
    _itot(toRead, increment, 10);
    _tcscpy(classpath, cPathPref);
    _tcscat(classpath, increment);

            // Get INI value
    reader.GetKeyValue(_T("Launcher"), classpath, _T(""), thisPath);
    if ( ! thisPath[0] )
      break;

            // Expands env vars
    // it now checks if there are two '%' not next to each other
    // ("%%" is not considered an environment var)
    TCHAR * firstPerc = _tcschr(thisPath, _T('%'));
    if ((firstPerc && firstPerc[1]) ? _tcschr(firstPerc + 2, _T('%')) : 0) 
        //so if firstPerc is NULL, or if the string finishes before firstPerc + 2,
        //it doesn't call strchr on a undefined string. If firstPerc isn't 0 neither
        //firstPerc[0] is, since it is a return value from strchr.
    {
      TCHAR lpEnvVarValue[Size];
      int nChars = ExpandEnvironmentStrings(thisPath, lpEnvVarValue, Size);

      if (nChars != 0 && _tcscmp(thisPath, lpEnvVarValue) != 0)
        _tcscpy(thisPath, lpEnvVarValue);
      //MODIFIED: if this envvar doesn't exist, and if it isn't a real path, the value is
      //discarded. So real paths including % are fully preserved.
      else if (! JPathFileExists(thisPath))
        continue;
    }
            //Splits the various files/dirs
    while (thisPath[0]) {
      size_t index = _tcscspn(thisPath, _T(";"));
      LPTSTR tString = new TCHAR[index + 1];
      _tcsncpy(tString, thisPath, index);                 //it doesn't copy the ';'
      tString[index] = NULL;                              //this IS required
            //Here tString is a single path
      resPaths.push_back(tString);
      thisPath += index;                                  //it now points to ';' or NULL
      if (thisPath[0])
        thisPath++;                                       //skips the ';'
    }
  }

  //delete[] classpath;

  *length = resPaths.size();
  LPTSTR *lpPaths = new LPTSTR[*length];
  for (int i = 0; i < *length; i++) {
    lpPaths[i] = resPaths[i];
  }
  return lpPaths;
}

// returns true if Jext is correctly installed

bool IsJextInstalled()
{
  bool installed = true;

  TCHAR outPath[MAX_PATH];
  TCHAR executionPath[MAX_PATH];
  GetModuleFileName(NULL, executionPath, MAX_PATH);
  PathRemoveFileSpec(executionPath);

  for (int i = 0; i < numOfClPaths; i++)
  {
    //(RE)MODIFIED: now no envvars in classPaths(if an envvar is not defined, it has been 
    // already discarded).
    //if (classPaths[i][0] != _T('%'))
    //{
      PathCombine(outPath, executionPath, classPaths[i]);
      installed = installed && JPathFileExists(outPath);
      if (!installed)
        break;
	//}
  }

  return (numOfClPaths == 0 ? false : installed);
}

//////////////////////////////////////////////////////////////////
// MAIN ENTRY POINT
//////////////////////////////////////////////////////////////////
//MODIFIED2: _tWinMain is the TCHAR version of WinMain
//so lpCmdLine is a LPTSTR

int APIENTRY _tWinMain(HINSTANCE hInstance,
                       HINSTANCE hPrevInstance,
                       LPTSTR    lpCmdLine,
                       int       nCmdShow)
{
  // GetKeyValue output strings;
  TCHAR outStr1[MAX_PATH], outStr2[MAX_PATH];

  //reads the classpaths from the .ini file for use by functions
  classPaths = GetClassPath(&numOfClPaths);
  // test if Jext is well installed
  if (!IsJextInstalled())
  {
    MessageBox(NULL,
               reader.GetKeyValue(_T("Messages"), _T("install.failed"), _T(""), outStr1),
               reader.GetKeyValue(_T("Messages"), _T("launcher.title"), _T("Launcher"), outStr2),
               MB_ICONEXCLAMATION);
    for (int i = 0; i < numOfClPaths; i++)
      delete[] classPaths[i];
    delete[] classPaths;
    return 1;
  }

  // try to launch a JRE
  LPCTSTR keys[] = { _T("Java Runtime Environment\\"),
                     _T("Java Development Kit\\")    };
  if (!Launch(keys, 2, lpCmdLine))
  {
      TCHAR jrePath[MAX_PATH];
      reader.GetKeyValue(_T("Launcher"), _T("jre"), _T(""), jrePath);

      // get current executing path
      // TCHAR executionPath[MAX_PATH];
      //GetCurrentDirectory((DWORD) MAX_PATH, executionPath);
      //PathCombine(jrePath, executionPath, jrePath);
      fullpath(jrePath, jrePath, MAX_PATH);

      // try to launch a local JRE
      if (!JPathFileExists(jrePath))
      {
        MessageBox(NULL,
                   reader.GetKeyValue(_T("Messages"), _T("no.jdk"), _T(""), outStr1),
                   reader.GetKeyValue(_T("Messages"), _T("launcher.title"), _T("Launcher"), outStr2),
                   MB_ICONEXCLAMATION);
        return 2;
      } else {
        // starts Jext with local JRE
        ExecuteJext(jrePath, lpCmdLine);
      }
  }
  
  for (int i = 0; i < numOfClPaths; i++)
    delete[] classPaths[i];
  delete[] classPaths;
  return 0;
}

// End of JextLauncher.cpp

