/*
 * 01/20/2003 - 23:17:11
 *
 * BundledSkinFactory.java - Provides bundled Skin's.
 * Copyright (C) 2002 Paolo Giarrusso
 * Tricks by Romain GUY
 * romain.guy@jext.org
 * blaisorblade_work@yahoo.it
 * www.jext.org
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jext.gui;

import javax.swing.UIManager;
import javax.swing.plaf.metal.MetalLookAndFeel;
import java.util.ArrayList;

class BundledSkinFactory implements SkinFactory
{
  public Skin[] getSkins() {
    ArrayList skins = new ArrayList(8);
    skins.add(new MetalSkin());
    skins.add(new JextSkin());
    skins.add(new GenericSkin("Unix Motif Skin", "motif", "com.sun.java.swing.plaf.motif.MotifLookAndFeel"));

    if (! UIManager.getSystemLookAndFeelClassName().equals( UIManager.getCrossPlatformLookAndFeelClassName()))
      skins.add(new GenericSkin("Native Skin", "native",  UIManager.getSystemLookAndFeelClassName()) );

    // these skins are added only if they are present on the underlying system
    // I use the same name since only one of the two will really exist.
    addSkinIfPresent(skins, "MacOs Native Skin", "_macos", "javax.swing.plaf.mac.MacLookAndFeel");
    addSkinIfPresent(skins, "MacOs Native Skin", "macos", "com.sun.java.swing.plaf.mac.MacLookAndFeel");

    //for windows Skin, it doesn't work, since the bytecode always exists.
    //So we override the standard isAvailable() method

    skins.add(new GenericSkin("Windows Native Skin", "windows",
          "com.sun.java.swing.plaf.windows.WindowsLookAndFeel") {
      public boolean isAvailable() {
        return (new com.sun.java.swing.plaf.windows.WindowsLookAndFeel()).isSupportedLookAndFeel();
      }
    });
    
    return (Skin[]) skins.toArray(new Skin[0]);
    //it passes a Skin[] to the toArray method;
    //otherwise the array would be an Object array with Skin's as elements;
    //so, instead it is at runtime a Skin array.
  }

  /**
   * Adds the descripted skin to the array list if the skins exists. The test is made against the
   * presence of the Look And Feel class on the system.
   * @return If the operation succeed, i.e. if the skin was added to the list
   */

  private boolean addSkinIfPresent(ArrayList skins, String description, String name, String lnfClass)
  {
    try
    {
      Class bytecode = Class.forName(lnfClass);
      if (bytecode != null)
      {
        skins.add(new GenericSkin(description, name, lnfClass));
        return true;
      }
    } catch (Exception e) { }

    return false;
  }

  private class MetalSkin extends Skin
  {
    public String getSkinName() {
      return "Standard Metal Skin";
    }
    public String getSkinInternName() {
      return "metal";
    }
    public void apply() throws Throwable {
      MetalLookAndFeel.setCurrentTheme(new javax.swing.plaf.metal.DefaultMetalTheme());
      UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
    }
  }
  
  private class JextSkin extends Skin
  {
    public String getSkinName() {
      return "Jext Metal Skin";
    }
    public String getSkinInternName() {
      return "jext";
    }
    public void apply() throws Throwable {
      MetalLookAndFeel.setCurrentTheme(new JextMetalTheme());
      UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
    }
  }
}
