/*
 * 11/12/2000 - 00:59:47
 *
 * AboutPlugins.java - Displays infos about plugins
 * Copyright (C) 2000 Romain Guy
 * romain.guy@jext.org
 * www.jext.org
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jext.misc;

import java.awt.*;
import java.awt.event.*;

import java.util.*;

import javax.swing.*;
import javax.swing.table.*;

import org.jext.*;
import org.jext.gui.*;

public class AboutPlugins extends JDialog implements ActionListener
{
  // private members
  private JextHighlightButton ok;
  private InstalledPlugin[] plugs;
  
  public AboutPlugins(JextFrame parent)
  {
    super(parent, Jext.getProperty("about.plugins.title"), false);
    getContentPane().setLayout(new BorderLayout());

    Box boxer = Box.createVerticalBox();
    JComponent box;

    ArrayList _plugins = JARClassLoader.pluginsNames;
    ArrayList _plugNames = JARClassLoader.pluginsRealNames;
    plugs = new InstalledPlugin[_plugNames.size()];

    if (plugs.length != 0)
    {
      String name, trueName;
      String[] args = new String[3];
      JextCheckBox _box;
      int startExt;
      boolean enabled;
  
      for (int i = 0; i < plugs.length; i++)
      {
	trueName = (String) _plugNames.get(i);//now the disabled property take names
	//from the real name(for KLNF.jar it's "KLNF") to avoid completely loading of such
	//plugins.
        name = (String) _plugins.get(i);
        startExt = name.indexOf(".class");//for enabled plugins, name should contain .class, but this doesn't
        //happen if the plugin has been enabled after Jext start.
        enabled = JARClassLoader.isEnabled(trueName);

	if (enabled && startExt != -1) {
	  int dot = name.lastIndexOf('/');
	  name = name.substring((dot == -1 ? 0 : dot + 1), startExt);
    
	  args[0] = Jext.getProperty("plugin." + name + ".name");
	  args[1] = Jext.getProperty("plugin." + name + ".version");
	  args[2] = Jext.getProperty("plugin." + name + ".author");
    
	  //JPanel pane = new JPanel();
    
	  _box = new JextCheckBox(Jext.getProperty("about.plugins.sentence", args));
	  //_box.setSelected("no".equals(Jext.getProperty("plugin." + trueName + ".disabled", "no")));
	  _box.setSelected(true);
	} else {//When plugins are not loaded we don't know their properties. So we
	  //use replacements. Ugly, but not loading is faster.
	  /*args[0] = trueName;
	  args[1] = "?";
	  args[2] = "?";
	  _box = new JextCheckBox(Jext.getProperty("about.plugins.sentence", args));*/
	  _box = new JextCheckBox(trueName);
	  _box.setSelected(enabled);
	}
	boxer.add(_box);
	plugs[i] = new InstalledPlugin(_box, trueName);
      }
  
      box = plugs[0].getCheckBox();
    } else {
      box = new JLabel(' ' + Jext.getProperty("no.plugins"));
      box.setForeground(Color.black);
      boxer.add(box);
    }

    getContentPane().add(new JLabel(Jext.getProperty("about.plugins.header")), BorderLayout.NORTH);
    JScrollPane scrollPane = new JScrollPane(boxer);
    FontMetrics fm = getFontMetrics(box.getFont());
    scrollPane.getViewport().setPreferredSize(new Dimension(30 * fm.charWidth('m'),
                                                            8 * box.getPreferredSize().height));
    getContentPane().add(scrollPane, BorderLayout.CENTER);

    ok = new JextHighlightButton(Jext.getProperty("general.ok.button"));
    ok.addActionListener(this);
    getRootPane().setDefaultButton(ok);

    JPanel pane = new JPanel();
    pane.add(ok);
    getContentPane().add(BorderLayout.SOUTH, pane);

    setDefaultCloseOperation(DISPOSE_ON_CLOSE);
    addKeyListener(new AbstractDisposer(this));

    pack();
    Utilities.centerComponentChild(parent, this);
    setVisible(true);
  }

  public void actionPerformed(ActionEvent evt)
  {
    if (evt.getSource() == ok)
    {
      for (int i = 0; i < plugs.length; i++)
        plugs[i].save();
      dispose();
    }
  }

  class InstalledPlugin
  {
    // private fields
    private String name;
    private JCheckBox box;
    
    /***************************************************************************
    Patch
     -> Memory management improvements : it may help the garbage collector.
     -> Author : Julien Ponge (julien@izforge.com)
     -> Date : 23, May 2001
     ***************************************************************************/
     protected void finalize() throws Throwable
     {
       super.finalize();

       name = null;
       box = null;
     }
     // End of patch

    InstalledPlugin(JCheckBox box, String name)
    {
      this.box = box;
      this.name = name;
    }

    public JCheckBox getCheckBox()
    {
      return box;
    }

    public void save()
    {
      //Jext.setProperty("plugin." + name + ".disabled", box.isSelected() ? "no" : "yes");
      JARClassLoader.setEnabled(name, box.isSelected());
    }
  }
  
  /***************************************************************************
  Patch
     -> Memory management improvements : it may help the garbage collector.
     -> Author : Julien Ponge (julien@izforge.com)
     -> Date : 23, May 2001
  ***************************************************************************/
  protected void finalize() throws Throwable
  {
    super.finalize();
    
    ok = null;
    plugs = null;
  }
  // End of patch
}

// End of AboutPlugins.java
