/*
 * 01/25/2003 - 13:42:28
 *
 * PythonLogWindow.java - Scripts log window
 * Copyright (C) 2003 Romain Guy
 * romain.guy@jext.org
 * www.jext.org
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jext.scripting.python;

import java.awt.BorderLayout;
import java.awt.Container;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseAdapter;

import javax.swing.*;
import javax.swing.text.Element;

import java.io.Writer;

import gnu.regexp.*;

import org.jext.*;
import org.jext.gui.*;

public class PythonLogWindow extends JFrame implements ActionListener
{
  private RE regexp;
  private JextFrame parent;
  private Container content;
  private boolean docked = false;
  private JextHighlightButton clear, dock;
  private JScrollPane textAreaScroller = null;
  private JTextArea textArea = new JTextArea(15, 40);

  public PythonLogWindow(JextFrame parent)
  {
    super(Jext.getProperty("python.window.title"));
    this.parent = parent;

    textArea.setEditable(false);
    textArea.addMouseListener(new MouseHandler());

    getContentPane().setLayout(new BorderLayout());
    getContentPane().add(BorderLayout.CENTER, new JScrollPane(textArea,
                                              ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
                                              ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS));

    JPanel pane = new JPanel(new BorderLayout());
    dock = new JextHighlightButton(Jext.getProperty("python.window.dock"));
    pane.add(BorderLayout.WEST, dock);
    pane.add(BorderLayout.CENTER, new JLabel(Jext.getProperty("python.window.advice")));
    clear = new JextHighlightButton(Jext.getProperty("python.window.clear"));
    pane.add(BorderLayout.EAST, clear);
    getContentPane().add(BorderLayout.SOUTH, pane);

    dock.addActionListener(this);
    clear.addActionListener(this);

    setDefaultCloseOperation(HIDE_ON_CLOSE);
    addKeyListener(new KeyAdapter()
    {
      public void keyPressed(KeyEvent evt)
      {
        if (evt.getKeyCode() == KeyEvent.VK_ESCAPE)
          setVisible(false);
      }
    });
    setIconImage(GUIUtilities.getJextIconImage());

    try
    {
      regexp = new RE("File \"([^\"]+)\", line (\\d+),.*");
    } catch (REException ree) {
      dispose();
    }

    pack();
    Utilities.centerComponent(this);
  }

  public void actionPerformed(ActionEvent evt)
  {
    Object o = evt.getSource();
    if (o == clear)
      textArea.setText("");
    else if (o == dock)
    {
      if (!docked)
      {
        dispose();
        //getContentPane().remove(textAreaScroller);
        parent.getVerticalTabbedPane().add(Jext.getProperty("python.window.tab"), content = getContentPane());
        dock.setLabel(Jext.getProperty("python.window.undock"));
        setContentPane(new JPanel());
        docked = true;
      } else {
        parent.getVerticalTabbedPane().remove(content);
        setContentPane(content);
        dock.setLabel(Jext.getProperty("python.window.dock"));
        SwingUtilities.invokeLater(new Runnable()
        {
          public void run()
          {
            setVisible(true);
            toFront();
            docked = false;
          }
        });
      }
    }
  }

  public boolean isDocked()
  {
    return docked;
  }

  public void log(String msg)
  {
    textArea.append(msg);
    textArea.setSelectionStart(textArea.getDocument().getLength());
    textArea.setSelectionEnd(textArea.getDocument().getLength());
  }

  public void logln(String msg)
  {
    this.log(msg + '\n');
  }

  class MouseHandler extends MouseAdapter
  {
    public void mouseClicked(MouseEvent evt)
    {
      if (evt.getClickCount() == 2)
      {
        try
        {
          Element map = textArea.getDocument().getDefaultRootElement();
          Element line = map.getElement(map.getElementIndex(textArea.getCaretPosition()));
          int start = line.getStartOffset();
          REMatch match = regexp.getMatch(textArea.getText(start, line.getEndOffset() - 1 - start));

          if (match != null)
          {
            // file
            String file = match.toString(1);
            // line number
            int lineNo = 0;
            // how can an exception be thrown here ? jeeezz... :)
            lineNo = Integer.parseInt(match.toString(2));
            JextTextArea _textArea = null;

            // script is executed from a text area
            if (file.equals("<string>"))
            {
              _textArea = parent.getTextArea();
            } else {
            // from a file

              JextTextArea[] areas = parent.getTextAreas();
              for (int i = 0; i < areas.length; i++)
              {
                if (file.equals(areas[i].getCurrentFile()))
                {
                  _textArea = areas[i];
                  break;
                }
              }

              if (_textArea == null)
                _textArea = parent.open(file, false);
            }

            // select error line
            line = _textArea.getDocument().getDefaultRootElement().getElement(lineNo - 1);
            if (line != null)
              _textArea.select(line.getStartOffset(), line.getEndOffset() - 1);
          }
        } catch (Exception e ) { }
      }
    }
  }

  private Writer writerStdOut = new Writer()
  {
    public void close() { }

    public void flush()
    {
      textArea.repaint();
    }

    public void write(char cbuf[], int off, int len)
    {
      log(new String(cbuf, off, len));
    }
  };

  private Writer writeStdErr = new Writer()
  {
    public void close() { }

    public void flush()
    {
      textArea.repaint();
    }

    public void write(char cbuf[], int off, int len)
    {
      log(new String(cbuf, off, len));
    }
  };

  /**
   * Returns a writer in which external classes can send
   * <code>String</code> to make them being displayed in the
   * console as standard output.
   */

  public Writer getStdOut()
  {
    return writerStdOut;
  }

  /**
   * Returns a writer in which external classes can send
   * <code>String</code> to make them being displayed in the
   * console as error output.
   */

  public Writer getStdErr()
  {
    return writeStdErr;
  }
}

// End of PythonLogWindow.java