/*
 * 01/25/2003 - 19:41:47
 *
 * Run.java
 * Copyright (C) 2001 Romain Guy
 *
 * This	free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jext.scripting.python;

import java.io.*;

import javax.swing.JOptionPane;
import java.awt.event.ActionEvent;

import org.jext.*;
import org.python.core.*;
import org.python.util.PythonInterpreter;

public final class Run
{
  private static PythonInterpreter parser;

  /**
   * Creates an interpreter.
   * @param parent The window which executes the script
   */

  public static PythonInterpreter getPythonInterpreter(JextFrame parent)
  {
    if (parser == null)
    {
      parser =  new PythonInterpreter();
      PyModule mod = imp.addModule("__main__");
      parser.setLocals(mod.__dict__);

			InputStream packages = Jext.class.getResourceAsStream("packages");
			
			if (packages != null)
			{
				PySystemState sys = Py.getSystemState();
				BufferedReader in = new BufferedReader(new InputStreamReader(packages));
				String line;
				try
				{
					while ((line = in.readLine()) != null)
						sys.add_package(line);
					in.close(); 
				} catch (IOException ioe) { }
			}//end if there is such a resource
    }

    parser.set("__jext__", parent);
    if (parent != null)
    {
      parser.setErr(parent.getPythonLogWindow().getStdErr());
      parser.setOut(parent.getPythonLogWindow().getStdOut());
    } else {
      parser.setOut(System.out);
      parser.setErr(System.err);
    }

    return parser;
  }

  /**
   * Evaluates some Python code.
   * @param code The script code to be evaluated
   * @param map A map of properties to add to interpreter
   * @param parent The window which executes the script
   * @return The result of the evaluation
   */

  public static PyObject eval(String code, String mapName, Object[] map, JextFrame parent)
  {
    try
    {
      PythonInterpreter parser = getPythonInterpreter(parent);

      if (map != null && mapName != null)
        parser.set(mapName, map);

      return parser.eval(code);
    } catch (Exception pe) {
      JOptionPane.showMessageDialog(parent, Jext.getProperty("python.script.errMessage"),
                                    Jext.getProperty("python.script.error"),
                                    JOptionPane.ERROR_MESSAGE);
      if (Jext.getBooleanProperty("dawn.scripting.debug"))
        System.err.println(pe.toString());
      // security ?
      parser = null;
    }

    return null;
  }

  /**
   * Executes some Python code.
   * @param code The script code to be interpreted
   * @param parent The window which executes the script
   */

  public static void execute(String code, JextFrame parent)
  {
    try
    {
      PythonInterpreter parser = getPythonInterpreter(parent);
      parser.exec(code);
    } catch (Exception pe) {
      JOptionPane.showMessageDialog(parent, Jext.getProperty("python.script.errMessage"),
                                    Jext.getProperty("python.script.error"),
                                    JOptionPane.ERROR_MESSAGE);
      if (Jext.getBooleanProperty("dawn.scripting.debug"))
        parent.getPythonLogWindow().logln(pe.toString());
      // security ?
      parser = null;
    }
  }

  /**
   * Runs a Jext script from a file.
   * @param fileName Path to the script
   * @param parent The Jext window which have to execute the script
   */

  public static void runScript(String fileName, JextFrame parent)
  {
    try
    {
      PythonInterpreter parser = getPythonInterpreter(parent);
      parser.execfile(fileName);
    } catch (Exception pe) {
      JOptionPane.showMessageDialog(parent, Jext.getProperty("python.script.errMessage"),
                                    Jext.getProperty("python.script.error"),
                                    JOptionPane.ERROR_MESSAGE);
      if (Jext.getBooleanProperty("dawn.scripting.debug"))
        parent.getPythonLogWindow().logln(pe.toString());
      // security ?
      parser = null;
    }
  }
}

// End of Run.java