/*
 * 03/23/2003
 *
 * Copyright (C) 2003 Paolo Giarrusso
 * blaisorblade_work@yahoo.it
 * www.jext.org
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import org.jext.misc.*;
import com.microstar.xml.*;
import java.io.Reader;
import java.util.*;
import java.net.URL;

public class PlugReader extends HandlerBase implements AbstractPlugReader {
  private boolean loaded = false;
  private PluginDesc[] list = null;

  public boolean loadXml(Reader in) {
    XmlParser parser = new XmlParser();
    parser.setHandler(this);
    try {
      parser.parse(null, null, in);
      loaded = true;
      return true;
    } catch (Exception e) {
      loaded = false;
      return false;
    }
  }

  public PluginDesc[] getPlugList() {
    if (!loaded)
      return null;
    return list;
  }
  private URL buildUrl(String fileName) {return null;}//must apply the template

  //XML parsing part.
  ArrayList plugins, authors, deps, files;
  PluginDesc currPlugin;
  String currUrlTemplate;
  StringBuffer content;
  HashMap attribs;

  public void startDocument() {
    plugins = new ArrayList(40);
    authors = new ArrayList(5);
    deps = new ArrayList(5);
    files = new ArrayList(3);
    attribs = new HashMap(6);
    content = new StringBuffer(600);
  }

  public void endDocument() {
    list = (PluginDesc[]) plugins.toArray(new PluginDesc[0]);
    plugins = null;
    attribs = null;
    content = null;
    authors = null;
    deps = null;
    files = null;
  }

  public void attribute(String atname, String value, boolean isSpecified) {
    attribs.put(atname, value);
  }

  public void startElement(String elname) {
    if (elname.equals("plugin")) {
      currPlugin = new PluginDesc((String) attribs.get("name"), (String) attribs.get("version"),
          (String) attribs.get("displayName"));
    } else if (elname.equals("dependStr")) {
      deps.add((String) attribs.get("value"));
    } else if (elname.equals("file")) {
      String type = (String) attribs.get("type");
      URL url = buildUrl((String) attribs.get("name"));
      int size = Integer.parseInt((String) attribs.get("size"));
      if (type.equals("bin"))
        currPlugin.setSrcUrl(url, size);
      else
        currPlugin.setBinUrl(url, size);
    //} else if (elname.equals("description")) {//?? not needed.
    } else if (elname.equals("author")) {
      authors.add(new PluginAuthor((String) attribs.get("email"), (String) attribs.get("name")));
    //} else if (elname.equals("htmlauthor")) {//?? not needed.
    } else if (elname.equals("plugList")) {
      currUrlTemplate = (String) attribs.get("downloadUrlTempl");
    }
  }

  public void charData(char[] ch, int start, int length) {
    content.append(ch, start, length);
  }

  public void endElement(String elname) {
    if (elname.equals("plugin")) {
      plugins.add(currPlugin);
      authors.clear();
      deps.clear();
    } else if (elname.equals("description"))
      currPlugin.setDesc(content.toString());
    else if (elname.equals("htmlauthor"))
      authors.add(new PluginAuthor(content.toString()));
    currPlugin = null;
    attribs.clear();
    content.setLength(0);
  }
}
